unit APIButton;

interface

uses
  Windows, Messages, Forms;

type
  TAPIButton = class;
  TButtonEvent = procedure(Sender: TAPIButton) of object;

  TAPIButton = class
  protected
    // varibles
    FHandle:THandle;
    FTop,FLeft,FWidth,FHeight:Integer;
    FMouseClick:TButtonEvent;
    OldWinProc,NewWinProc:Pointer;
    FMouse:Boolean;
    // for 'Caption' property
    function GetCaption:string;
    procedure SetCaption(const Value: string);
    // for 'Left' property
    procedure SetLeft(const Value: Integer);
    // for 'Top' property
    procedure SetTop(const Value: Integer);
    // for 'Width' property
    procedure SetWidth(const Value: Integer);
    // for 'Height' property
    procedure SetHeight(const Value: Integer);
    // for 'Visible' property
    function GetVisible:Boolean;
    procedure SetVisible(const Value: Boolean);
    // for 'Enabled' property
    function GetEnabledB:Boolean;
    procedure SetEnabledB(const Value: Boolean);
    // for 'Parent' property
    function GetParent:THandle;
    procedure SetParent(const Value: THandle);
    // for Windows Messages
    procedure msgUser(var Msg:TMessage);
  published
    property Handle: THandle read FHandle default 0;
    property Caption:string read GetCaption write SetCaption;
    property Left:Integer read FLeft write SetLeft;
    property Top:Integer read FTop write SetTop;
    property Width:Integer read FWidth write SetWidth;
    property Height:Integer read FHeight write SetHeight;
    property Visible:Boolean read GetVisible write SetVisible;
    property Enabled:Boolean read GetEnabledB write SetEnabledB;
    property Parent:THandle read GetParent write SetParent;
    property OnMouseClick:TButtonEvent read FMouseClick write FMouseClick;
    procedure SetFocus;
    procedure SetBounds(ALeft,ATop,AWidth,AHeight: Integer);
    constructor Create(ParentControl: THandle);
    destructor UnCreate;
  end;

implementation

{ TAPIButton }

constructor TAPIButton.Create(ParentControl: THandle);
begin
   FHandle:=CreateWindow('Button','APIButton',WS_VISIBLE or WS_CHILD,0,0,72,25,
               ParentControl,0,hInstance,nil);
   SendMessage(FHandle,WM_CREATE,0,0);
   FMouse:=GetSystemMetrics(SM_SWAPBUTTON) = 1;
   FLeft:=0;
   FTop:=0;
   FWidth:=72;
   FHeight:=25;
   NewWinProc:=MakeObjectInstance(msgUser);
   OldWinProc:=Pointer(SetWindowLong(FHandle,GWL_WNDPROC,Cardinal(NewWinProc)));
end;

destructor TAPIButton.UnCreate;
begin
   Visible:=False;
   SendMessage(FHandle,WM_DESTROY,0,0);
end;

function TAPIButton.GetCaption: string;
var
   Buf:string;
   L:Integer;
begin
   L:=GetWindowTextLength(FHandle) + 1;
   SetLength(Buf,L);
   GetWindowText(FHandle,PChar(Buf),L);
   Result:=PChar(Buf);
end;

function TAPIButton.GetEnabledB: Boolean;
begin
   Result:=IsWindowEnabled(FHandle);
end;

function TAPIButton.GetVisible: Boolean;
begin
   Result:=IsWindowVisible(FHandle);
end;

procedure TAPIButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
   FLeft:=ALeft;
   FTop:=ATop;
   FWidth:=AWidth;
   FHeight:=AHeight;
   SetWindowPos(FHandle, 0, ALeft, ATop, AWidth, AHeight,0)
end;

procedure TAPIButton.SetCaption(const Value: string);
begin
   SetWindowText(FHandle,PChar(Value));
end;

procedure TAPIButton.SetEnabledB(const Value: Boolean);
begin
   EnableWindow(FHandle,Value);
end;

procedure TAPIButton.SetFocus;
begin
   Windows.SetFocus(FHandle);
end;

procedure TAPIButton.SetHeight(const Value: Integer);
begin
   FHeight:=Value;
   SetWindowPos(FHandle,0,FLeft,FTop,FWidth,Value,0);
end;

procedure TAPIButton.SetLeft(const Value: Integer);
begin
   FLeft:=Value;
   SetWindowPos(FHandle,0,Value,FTop,FWidth,FHeight,0);
end;

procedure TAPIButton.SetWidth(const Value: Integer);
begin
   FWidth:=Value;
   SetWindowPos(FHandle,0,FLeft,FTop,Value,FHeight,0);
end;

procedure TAPIButton.SetTop(const Value: Integer);
begin
   FTop:=Value;
   SetWindowPos(FHandle,0,FLeft,Value,FWidth,FHeight,0);
end;

procedure TAPIButton.SetVisible(const Value: Boolean);
begin
   if Value then
      ShowWindow(FHandle,SW_SHOW)
   else
      ShowWindow(FHandle,SW_HIDE);
end;

function TAPIButton.GetParent: THandle;
begin
   Result:=Windows.GetParent(FHandle);
end;

procedure TAPIButton.SetParent(const Value: THandle);
begin
   Windows.SetParent(FHandle, Value);
end;

procedure TAPIButton.msgUser(var Msg: TMessage);
   procedure RaiseClick;
   var
      Pnt:TPoint;
   begin
      GetCursorPos(Pnt);
      if Assigned(FMouseClick) and (WindowFromPoint(Pnt) = FHandle) then
         FMouseClick(Self);
   end;
begin
   Msg.Result:=CallWindowProc(OldWinProc,FHandle,Msg.Msg,Msg.WParam,Msg.LParam);
   if not FMouse and (Msg.Msg = Wm_LButtonUp) then
      RaiseClick
   else if FMouse and (Msg.Msg = Wm_RButtonUp) then
      RaiseClick;
end;

end.
